import json, requests

from . import config as c
from . import hub

cloudBase='https://cloud2.cozify.fi/ui/0.2/'

# auth flow based on and storing into config
# email -> OTP -> remoteToken -> hub ip -> hubToken
def authenticate():
    if 'email' not in  c.ephemeral['Cloud'] or not  c.ephemeral['Cloud']['email']:
         c.ephemeral['Cloud']['email'] = _getEmail()
         c.ephemeralWrite()
    email = c.ephemeral['Cloud']['email']

    if _needRemoteToken():
        if _requestlogin(email):
            otp = _getotp()
            remoteToken = _emaillogin(email, otp)
            if remoteToken is not None:
                c.ephemeral['Cloud']['remoteToken'] = remoteToken
                c.ephemeralWrite()
            else:
                # remoteToken fail
                print('remoteToken failed')
                return False
        else:
            # requestlogin fail
            print('requestlogin failed')
            return False
    else:
        # remoteToken already fine, let's just use it
        remoteToken = c.ephemeral['Cloud']['remoteToken']

    if _needHubToken():
        hubIps = _lan_ip(remoteToken)
        hubkeys = _hubkeys(remoteToken)
        if hubIps is not None and hubkeys is not None:
            for hubIp in hubIps: # hubIps is returned as a list of all hubs
                hubMap = hub._hub(remoteToken, hubIp)
                if hubMap is not None:
                    hubId = hubMap['hubId']
                    hubName = hubMap['name']
                    hubToken = hubkeys[hubId]
                    if hubToken:
                        if 'Hubs.' + hubName not in c.ephemeral:
                            c.ephemeral['Hubs.' + hubName] = {}
                        if 'default' not in c.ephemeral['Hubs']:
                            c.ephemeral['Hubs']['default'] = hubName 

                        c.ephemeral['Hubs.' + hubName]['hubToken'] = hubToken
                        c.ephemeral['Hubs.' + hubName]['host'] = hubIp
                        c.ephemeral['Hubs.' + hubName]['hubId'] = hubId # not really used for anything but doesn't hurt
                        c.ephemeralWrite()
                    else:
                        # hubToken fail
                        print('hubToken failed')
                        return False
                else:
                    # hubmap fail
                    print('hubmap failed')
                    return False
        else:
            # lan_ip/hubkeys fail
            print('lan_ip or hubkeys failed')
            return False
    return True



# check if we currently hold a remoteKey.
# TODO(artanicus): need to do an OPTIONS call to check validity as well
def _needRemoteToken():
    # check if we've got a valid remoteToken
    if 'remoteToken' in c.ephemeral['Cloud']:
        if c.ephemeral['Cloud']['remoteToken'] is not None:
            return False
    return True

def _needHubToken():
    # this is a complex issue, for now just return a naive if default hub key is there, assume it's good
    if 'default' not in c.ephemeral['Hubs'] or 'hubtoken' not in c.ephemeral['Hubs.' + c.ephemeral['Hubs']['default']]:
        return True
    else:
        return False

def _getotp():
    return input('OTP from your email: ')

def _getEmail():
    return input('Enter your Cozify account email address: ')

# 1:1 implementation of user/requestlogin
# email: cozify account email
# returns success Bool
def _requestlogin(email):
    payload = { 'email': email }
    response = requests.post(cloudBase + 'user/requestlogin', params=payload)
    if response.status_code == 200:
        return True
    else:
        print(response.text)
        return False

# 1:1 implementation of user/emaillogin
# email: cozify account email
# otp: OTP provided by user, generated by requestlogin
# returns remote-key or None on failure
def _emaillogin(email, otp):
    payload = {
            'email': email,
            'password': otp
    }

    response = requests.post(cloudBase + 'user/emaillogin', params=payload)
    if response.status_code == 200:
        return response.text
    else:
        print(response.text)
        return None

# 1:1 implementation of hub/lan_ip
# remoteToken: cozify Cloud remoteToken
# returns list of hub ip's or None
def _lan_ip(remoteToken):
    headers = {
            'Authorization': remoteToken
    }

    response = requests.get(cloudBase + 'hub/lan_ip', headers=headers)
    if response.status_code == 200:
        return json.loads(response.text)
    else:
        print(response.text)
        return None

# 1:1 implementation of user/hubkeys
# remoteToken: cozify Cloud remoteToken
# returns map of hubs: { hubId: hubToken }
def _hubkeys(remoteToken):
    headers = {
            'Authorization': remoteToken
    }

    response = requests.get(cloudBase + 'user/hubkeys', headers=headers)
    if response.status_code == 200:
        return json.loads(response.text)
    else:
        print(response.text)
        return None
